﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Web;

using Tea.Utils.Extension;
using TeaCommerce.Data.Payment;
using umbraco.BusinessLogic;
using TeaCommerce.Data;

namespace estore.grantadesign.com.classes
{
    public class SecureTrading : APaymentProvider
    {
        protected const string defaultParameterValue = "No value";
        public override string FormPostUrl { get { return "https://payments.securetrading.net/process/payments/choice"; } }

        public override bool AllowsCancelPayment { get { return false; } }
        public override bool AllowsCapturePayment { get { return false; } }
        public override bool AllowsRefundPayment { get { return false; } }
        public override bool AllowsGetStatus { get { return false; } }
        public override bool AllowCallbackWithoutOrderId { get { return true; } }

        public override Dictionary<string, string> DefaultSettings
        {
            get
            {
                if (defaultSettings == null)
                {
                    defaultSettings = new Dictionary<string, string>();
                    defaultSettings["sitereference"] = string.Empty;
                    defaultSettings["continueurl"] = string.Empty;
                    defaultSettings["cancelurl"] = string.Empty;
                    defaultSettings["streetAddressPropertyAlias"] = "streetAddress";
                    defaultSettings["cityPropertyAlias"] = "city";
                    defaultSettings["countyPropertyAlias"] = "state";
                    defaultSettings["zipCodePropertyAlias"] = "zipCode";
                    defaultSettings["telephonePropertyAlias"] = "phone";
                    defaultSettings["version"] = "1";
                    defaultSettings["md5AuthSecret"] = string.Empty;
                }
                return defaultSettings;
            }
        }

        public override Dictionary<string, string> GenerateForm(Order order, string teaCommerceContinueUrl, string teaCommerceCancelUrl, string teaCommerceCallBackUrl, Dictionary<string, string> settings)
        {
            List<string> settingsToExclude = new string[] { "md5AuthSecret", "continueurl", "cancelurl", "streetAddressPropertyAlias", "cityPropertyAlias", "countyPropertyAlias", "zipCodePropertyAlias", "telephonePropertyAlias" }.ToList();
            Dictionary<string, string> inputFields = settings.Where(i => !settingsToExclude.Contains(i.Key)).ToDictionary(i => i.Key, i => i.Value);

            OrderProperty streetAddressProperty = settings.ContainsKey("streetAddressPropertyAlias") ? order.Properties.FirstOrDefault(i => i.Alias.Equals(settings["streetAddressPropertyAlias"])) : null;
            string streetAddress = streetAddressProperty != null && !string.IsNullOrEmpty(streetAddressProperty.Value) ? streetAddressProperty.Value : defaultParameterValue;

            OrderProperty cityProperty = settings.ContainsKey("cityPropertyAlias") ? order.Properties.FirstOrDefault(i => i.Alias.Equals(settings["cityPropertyAlias"])) : null;
            string city = cityProperty != null && !string.IsNullOrEmpty(cityProperty.Value) ? cityProperty.Value : defaultParameterValue;

            OrderProperty countyProperty = settings.ContainsKey("countyPropertyAlias") ? order.Properties.FirstOrDefault(i => i.Alias.Equals(settings["countyPropertyAlias"])) : null;
            string county = countyProperty != null && !string.IsNullOrEmpty(countyProperty.Value) ? countyProperty.Value : defaultParameterValue;

            OrderProperty zipCodeProperty = settings.ContainsKey("zipCodePropertyAlias") ? order.Properties.FirstOrDefault(i => i.Alias.Equals(settings["zipCodePropertyAlias"])) : null;
            string zipCode = zipCodeProperty != null && !string.IsNullOrEmpty(zipCodeProperty.Value) ? zipCodeProperty.Value : defaultParameterValue;

            OrderProperty telephoneProperty = settings.ContainsKey("telephonePropertyAlias") ? order.Properties.FirstOrDefault(i => i.Alias.Equals(settings["telephonePropertyAlias"])) : null;
            string telephone = telephoneProperty != null && !string.IsNullOrEmpty(telephoneProperty.Value) ? telephoneProperty.Value : defaultParameterValue;

            string siteReference = settings["sitereference"];
            inputFields["sitereference"] = siteReference;

            //order id
            inputFields["orderreference"] = order.Name;

            //currency
            inputFields["currencyiso3a"] = order.CurrencyISOCode;

            //amount
            string amount = order.TotalPrice.ToString("0.00", CultureInfo.InvariantCulture);
            inputFields["mainamount"] = amount;

            //sitesecurity
            if (settings.ContainsKey("md5AuthSecret") && !string.IsNullOrEmpty(settings["md5AuthSecret"]))
                inputFields["sitesecurity"] = GetMD5Hash(siteReference + order.CurrencyISOCode + amount + settings["md5AuthSecret"]);

            //address
            inputFields["billinglastname"] = order.LastName.Truncate(20);
            inputFields["billingfirstname"] = order.FirstName.Truncate(20);
            inputFields["billingstreet"] = streetAddress.Truncate(100);
            inputFields["billingtown"] = city.Truncate(40);
            inputFields["billingcounty"] = county.Truncate(40);
            inputFields["billingpostcode"] = zipCode.Truncate(10);
            inputFields["billingcountryiso2a"] = order.Country.CountryCode;
            inputFields["billingemail"] = order.Email.Truncate(100);
            inputFields["billingtelephone"] = telephone.Truncate(20);
            inputFields["billingtelephonetype"] = "W";

            //Secure Trading doesn't support showing order line information.

            return inputFields;
        }

        public override CallbackInfo ProcessCallback(Order order, HttpRequest request, Dictionary<string, string> settings)
        {
            try
            {
                string errorMessage = string.Empty;
                string errorcode = request.Form["errorcode"];

                if (errorcode.Equals("0"))
                {
                    string sitereference = request.Form["sitereference"];
                    string currencyiso3a = request.Form["currencyiso3a"];
                    string mainamount = request.Form["mainamount"];

                    string md5CheckValue = string.Empty;
                    if (settings.ContainsKey("md5AuthSecret") && !string.IsNullOrEmpty(settings["md5AuthSecret"]))
                        md5CheckValue = GetMD5Hash(sitereference + currencyiso3a + mainamount + settings["md5AuthSecret"]);

                    if (md5CheckValue == string.Empty || md5CheckValue.Equals(request.Form["responsesitesecurity"]))
                    {
                        string transactionreference = request.Form["transactionreference"];
                        string paymenttypedescription = request.Form["paymenttypedescription"];
                        string maskedpan = request.Form["maskedpan"];
                        string status = request.Form["status"];

                        PaymentStatus paymentStatus;

                        switch (status)
                        {
                            case "0":
                            case "1":
                                paymentStatus = PaymentStatus.PendingExternalSystem;
                                break;
                            case "2":
                            case "55":
                                paymentStatus = PaymentStatus.Error;
                                break;
                            case "3":
                                paymentStatus = PaymentStatus.Cancelled;
                                break;
                            case "100":
                                paymentStatus = PaymentStatus.Captured;
                                break;
                            default:
                                paymentStatus = PaymentStatus.Authorized;
                                break;
                        }

                        decimal totalAmount = (mainamount.ParseToDecimal(CultureInfo.InvariantCulture, 0));

                        return new CallbackInfo(order.Name, totalAmount, transactionreference, paymentStatus, paymenttypedescription, maskedpan);
                    }
                    else
                        errorMessage = "Tea Commerce - Secure Trading - MD5Sum security check failed";
                }
                else
                    errorMessage = "Tea Commerce - Secure Trading - Payment not approved: " + errorcode;

                Log.Add(LogTypes.Error, -1, errorMessage);
                return new CallbackInfo(errorMessage);
            }
            catch (Exception ex)
            {
                Log.Add(LogTypes.Error, -1, "SecureTrading.ProcessCallback: " + ex.Message);
                throw;
            }
        }

        public override string GetContinueUrl(Dictionary<string, string> settings)
        {
            return settings["continueurl"];
        }

        public override string GetCancelUrl(Dictionary<string, string> settings)
        {
            return settings["cancelurl"];
        }

        public override long? GetOrderId(HttpRequest request, Dictionary<string, string> settings)
        {
            try
            {
                string errorMessage = string.Empty;
                string errorcode = request.Form["errorcode"];

                if (errorcode.Equals("0"))
                {
                    string sitereference = request.Form["sitereference"];
                    string currencyiso3a = request.Form["currencyiso3a"];
                    string mainamount = request.Form["mainamount"];

                    string md5CheckValue = string.Empty;
                    if (settings.ContainsKey("md5AuthSecret") && !string.IsNullOrEmpty(settings["md5AuthSecret"]))
                        md5CheckValue = GetMD5Hash(sitereference + currencyiso3a + mainamount + settings["md5AuthSecret"]);

                    if (md5CheckValue == string.Empty || md5CheckValue.Equals(request.Form["responsesitesecurity"]))
                    {
                        string orderID = request.Form["orderreference"];
                        if (TeaCommerceSettings.OrderNamePrefix != string.Empty)
                        {
                            orderID = orderID.Remove(0, TeaCommerceSettings.OrderNamePrefix.Length);
                            // Older versions of Tea Commerce used to auto append a '-'.
                            orderID = orderID.Replace("-", string.Empty);
                        }
                       
                        return orderID.ParseToLong();
                    }
                    else
                        errorMessage = "Tea Commerce - Secure Trading - MD5Sum security check failed";
                }
                else                    
                    errorMessage = "Tea Commerce - Secure Trading - Payment not approved: " + errorcode;                

                Log.Add(LogTypes.Error, -1, errorMessage);
                return null;
            }
            catch (Exception ex)
            {
                Log.Add(LogTypes.Error, -1, "SecureTrading.GetOrderId: " + ex.Message);
                throw;
            }
        }

        public override APIInfo GetStatus(Order order, Dictionary<string, string> settings)
        {
            throw new NotSupportedException();
        }

        public override APIInfo CapturePayment(Order order, Dictionary<string, string> settings)
        {
            throw new NotImplementedException();
        }

        public override APIInfo RefundPayment(Order order, Dictionary<string, string> settings)
        {
            throw new NotImplementedException();
        }

        public override APIInfo CancelPayment(Order order, Dictionary<string, string> settings)
        {
            throw new NotImplementedException();
        }
    }
}